@@ -24,15 +24,15 @@ CK_UNAVAILABLE_INFORMATION = PyKCS11.LowLevel.CK_UNAVAILABLE_INFORMATION
 CK_EFFECTIVELY_INFINITE = PyKCS11.LowLevel.CK_EFFECTIVELY_INFINITE
 CK_INVALID_HANDLE = PyKCS11.LowLevel.CK_INVALID_HANDLE
 
-CKM = {};
-CKR = {};
-CKA = {};
-CKO = {};
-CKU = {};
-CKK = {};
-CKC = {};
-CKF = {};
-CKS = {};
+CKM = {}
+CKR = {}
+CKA = {}
+CKO = {}
+CKU = {}
+CKK = {}
+CKC = {}
+CKF = {}
+CKS = {}
 
 # redefine PKCS#11 constants using well known prefixes
 for x in PyKCS11.LowLevel.__dict__.keys():
@@ -44,18 +44,18 @@ for x in PyKCS11.LowLevel.__dict__.keys():
       or x[:4] == 'CKK_' \
       or x[:4] == 'CKC_' \
       or x[:4] == 'CKF_' \
-      or x[:4] == 'CKS_' \
-      :
-        a = "%s=PyKCS11.LowLevel.%s" % (x, x) 
+      or x[:4] == 'CKS_':
+        a = "%s=PyKCS11.LowLevel.%s" % (x, x)
         exec(a)
         if x[3:] != "_VENDOR_DEFINED":
-            eval(x[:3])[eval(x)] = x # => CKM[CKM_RSA_PKCS] = 'CKM_RSA_PKCS'
-            eval(x[:3])[x] = eval(x) # => CKM['CKM_RSA_PKCS'] = CKM_RSA_PKCS
+            eval(x[:3])[eval(x)] = x  # => CKM[CKM_RSA_PKCS] = 'CKM_RSA_PKCS'
+            eval(x[:3])[x] = eval(x)  # => CKM['CKM_RSA_PKCS'] = CKM_RSA_PKCS
 
 # special CKR[] values
 CKR[-2] = "Unkown PKCS#11 type"
 CKR[-1] = "Load"
 
+
 class CK_SLOT_INFO(object):
     """
     matches the PKCS#11 CK_SLOT_INFO structure
@@ -75,8 +75,7 @@ class CK_SLOT_INFO(object):
     flags_dict = {
         CKF_TOKEN_PRESENT: "CKF_TOKEN_PRESENT",
         CKF_REMOVABLE_DEVICE: "CKF_REMOVABLE_DEVICE",
-        CKF_HW_SLOT: "CKF_HW_SLOT"
-    }
+        CKF_HW_SLOT: "CKF_HW_SLOT"}
 
     def flags2text(self):
         """
@@ -92,6 +91,7 @@ class CK_SLOT_INFO(object):
                 r.append(CK_SLOT_INFO.flags_dict[v])
         return r
 
+
 class CK_INFO(object):
     """
     matches the PKCS#11 CK_INFO structure
@@ -108,6 +108,7 @@ class CK_INFO(object):
     @type libraryVersion: list
     """
 
+
 class CK_SESSION_INFO(object):
     """
     matches the PKCS#11 CK_SESSION_INFO structure
@@ -151,6 +152,7 @@ class CK_SESSION_INFO(object):
         """
         return CKS[self.state]
 
+
 class CK_TOKEN_INFO(object):
     """
     matches the PKCS#11 CK_TOKEN_INFO structure
@@ -228,6 +230,7 @@ class CK_TOKEN_INFO(object):
                 r.append(CK_TOKEN_INFO.flags_dict[v])
         return r
 
+
 class CK_MECHANISM_INFO(object):
     """
     matches the PKCS#11 CK_MECHANISM_INFO structure
@@ -271,10 +274,11 @@ class CK_MECHANISM_INFO(object):
                 r.append(CK_MECHANISM_INFO.flags_dict[v])
         return r
 
+
 class PyKCS11Error(Exception):
     """ define the possible PKCS#11 error codes """
 
-    def __init__(self, value, text = ""):
+    def __init__(self, value, text=""):
         self.value = value
         self.text = text
 
@@ -288,6 +292,7 @@ class PyKCS11Error(Exception):
         else:
             return CKR[self.value] + " (0x%08X)" % self.value
 
+
 class PyKCS11Lib(object):
     """ high level PKCS#11 binding """
 
@@ -297,7 +302,7 @@ class PyKCS11Lib(object):
     def __del__(self):
         self.lib.Unload()
 
-    def load(self, pkcs11dll_filename = None, *init_string):
+    def load(self, pkcs11dll_filename=None, *init_string):
         """
         load a PKCS#11 library
 
@@ -381,7 +386,7 @@ class PyKCS11Lib(object):
         @type slot: integer
         @return: a L{CK_TOKEN_INFO} object
         """
-        tokeninfo =  PyKCS11.LowLevel.CK_TOKEN_INFO()
+        tokeninfo = PyKCS11.LowLevel.CK_TOKEN_INFO()
         rv = self.lib.C_GetTokenInfo(slot, tokeninfo)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
@@ -427,12 +432,14 @@ class PyKCS11Lib(object):
 
         return t
 
-    def openSession(self, slot, flags = 0):
+    def openSession(self, slot, flags=0):
         """
         C_OpenSession
 
         @param slot: slot number returned by L{getSlotList}
         @type slot: integer
+        @param flags: 0 (default), CKF_RW_SESSION for RW session
+        @type flags: integer
         @return: a L{Session} object
         """
         se = PyKCS11.LowLevel.CK_SESSION_HANDLE()
@@ -483,8 +490,26 @@ class PyKCS11Lib(object):
 
         return i
 
+    def waitForSlotEvent(self, flags=0):
+        """
+        C_WaitForSlotEvent
+
+        @param flags: 0 (default) or CKF_DONT_BLOCK
+        @type flags: integer
+        @return: slot
+        @rtype: integer
+        """
+        tmp = 0
+        (rv, slot) = self.lib.C_WaitForSlotEvent(flags, tmp)
+        if rv != CKR_OK:
+            raise PyKCS11Error(rv)
+
+        return slot
+
+
 class Mechanism(object):
     """Wraps CK_MECHANISM"""
+
     def __init__(self, mechanism, param):
         """
         @param mechanism: the mechanism to be used
@@ -492,7 +517,7 @@ class Mechanism(object):
         @param param: data to be used as crypto operation parameter
         (i.e. the IV for some agorithms)
         @type param: string or list/tuple of bytes
-        
+
         @see: L{Session.decrypt}, L{Session.sign}
         """
         self.mechanism = mechanism
@@ -500,6 +525,7 @@ class Mechanism(object):
 
 MechanismRSAPKCS1 = Mechanism(CKM_RSA_PKCS, None)
 
+
 class Session(object):
     """ Manage L{PyKCS11Lib.openSession} objects """
 
@@ -537,7 +563,7 @@ class Session(object):
         s.ulDeviceError = sessioninfo.ulDeviceError
         return s
 
-    def login(self, pin, user_type = CKU_USER):
+    def login(self, pin, user_type=CKU_USER):
         """
         C_Login
 
@@ -584,29 +610,29 @@ class Session(object):
         rv = self.lib.C_SetPIN(self.session, old_pin, new_pin)
         if rv != CKR_OK:
             raise PyKCS11Error(rv)
-            
+
     def sign(self, key, data, mecha=MechanismRSAPKCS1):
         """
         C_SignInit/C_Sign
-        
+
         @param key: a key handle, obtained calling L{findObjects}.
         @type key: integer
         @param data: the data to be signed
         @type data:  (binary) sring or list/tuple of bytes
         @param mecha: the signing mechanism to be used
-        @type mecha: L{Mechanism} instance or L{MechanismRSAPKCS1} 
+        @type mecha: L{Mechanism} instance or L{MechanismRSAPKCS1}
         for CKM_RSA_PKCS
         @return: the computed signature
         @rtype: list of bytes
-        
+
         @note: the returned value is an istance of L{LowLevel.ckbytelist}.
         You can easly convert it to a binary string with::
             ''.join(chr(i) for i in ckbytelistSignature)
-        
+
         """
         m = PyKCS11.LowLevel.CK_MECHANISM()
         signature = PyKCS11.LowLevel.ckbytelist()
-        ba = None # must be declared here or may be deallocated too early
+        ba = None  # must be declared here or may be deallocated too early
         m.mechanism = mecha.mechanism
         if (mecha.param):
             ba = PyKCS11.LowLevel.byteArray(len(mecha.param))
@@ -616,7 +642,8 @@ class Session(object):
             else:
                 for c in xrange(len(mecha.param)):
                     ba[c] = mecha.param[c]
-            # with cast() the ba object continue to own internal pointer (avoids a leak).
+            # with cast() the ba object continue to own internal pointer
+            # (avoids a leak).
             # pParameter is an opaque pointer, never garbage collected.
             m.pParameter = ba.cast()
             m.ulParameterLen = len(mecha.param)
@@ -632,19 +659,19 @@ class Session(object):
         if (rv != 0):
             raise PyKCS11Error(rv)
         #first call get signature size
-        rv = self.lib.C_Sign(self.session, data1, signature);
+        rv = self.lib.C_Sign(self.session, data1, signature)
         if (rv != 0):
             raise PyKCS11Error(rv)
         #second call get actual signature data
-        rv = self.lib.C_Sign(self.session, data1, signature);
+        rv = self.lib.C_Sign(self.session, data1, signature)
         if (rv != 0):
             raise PyKCS11Error(rv)
         return signature
-        
+
     def decrypt(self, key, data, mecha=MechanismRSAPKCS1):
         """
         C_DecryptInit/C_Decrypt
-        
+
         @param key: a key handle, obtained calling L{findObjects}.
         @type key: integer
         @param data: the data to be decrypted
@@ -654,15 +681,15 @@ class Session(object):
         for CKM_RSA_PKCS
         @return: the decrypted data
         @rtype: list of bytes
-        
+
         @note: the returned value is an istance of L{LowLevel.ckbytelist}.
         You can easly convert it to a binary string with::
             ''.join(chr(i) for i in ckbytelistData)
-        
+
         """
         m = PyKCS11.LowLevel.CK_MECHANISM()
         decrypted = PyKCS11.LowLevel.ckbytelist()
-        ba = None # must be declared here or may be deallocated too early
+        ba = None  # must be declared here or may be deallocated too early
         m.mechanism = mecha.mechanism
         if (mecha.param):
             ba = PyKCS11.LowLevel.byteArray(len(mecha.param))
@@ -672,7 +699,8 @@ class Session(object):
             else:
                 for c in xrange(len(mecha.param)):
                     ba[c] = mecha.param[c]
-            # with cast() the ba object continue to own internal pointer (avoids a leak).
+            # with cast() the ba object continue to own internal pointer
+            # (avoids a leak).
             # pParameter is an opaque pointer, never garbage collected.
             m.pParameter = ba.cast()
             m.ulParameterLen = len(mecha.param)
@@ -688,20 +716,21 @@ class Session(object):
         if (rv != 0):
             raise PyKCS11Error(rv)
         #first call get decrypted size
-        rv = self.lib.C_Decrypt(self.session, data1, decrypted);
+        rv = self.lib.C_Decrypt(self.session, data1, decrypted)
         if (rv != 0):
             raise PyKCS11Error(rv)
         #second call get actual decrypted data
-        rv = self.lib.C_Decrypt(self.session, data1, decrypted);
+        rv = self.lib.C_Decrypt(self.session, data1, decrypted)
         if (rv != 0):
             raise PyKCS11Error(rv)
         return decrypted
-        
+
     def isNum(self, type):
         if type in (CKA_CERTIFICATE_TYPE,
             CKA_CLASS,
             CKA_KEY_GEN_MECHANISM,
             CKA_KEY_TYPE,
+            CKA_MODULUS_BITS,
             CKA_VALUE_BITS,
             CKA_VALUE_LEN):
             return True
@@ -739,7 +768,7 @@ class Session(object):
     def isBin(self, type):
         return (not self.isBool(type)) and (not self.isString(type)) and (not self.isNum(type))
 
-    def findObjects(self, template = ()):
+    def findObjects(self, template=()):
         """
         find the objects matching the template pattern
 
@@ -772,7 +801,8 @@ class Session(object):
         while True:
             self.lib.C_FindObjects(self.session, result)
             for x in result:
-                # make a copy of the handle: the original value get corrupted (!!)
+                # make a copy of the handle: the original value get
+                # corrupted (!!)
                 a = PyKCS11.LowLevel.CK_OBJECT_HANDLE()
                 a.assign(x.value())
                 res.append(a)
@@ -782,7 +812,7 @@ class Session(object):
         self.lib.C_FindObjectsFinal(self.session)
         return res
 
-    def getAttributeValue(self, obj_id, attr, allAsBinary = False):
+    def getAttributeValue(self, obj_id, attr, allAsBinary=False):
         """
         C_GetAttributeValue
 
@@ -794,16 +824,16 @@ class Session(object):
         @type allAsBinary: Boolean
         @return: a list of values corresponding to the list of attributes
         @rtype: list
-        
+
         @see: L{getAttributeValue_fragmented}
-        
+
         @note: if allAsBinary is True the function do not convert results to
         Python types (i.e.: CKA_TOKEN to Bool, CKA_CLASS to int, ...).
         Binary data is returned as L{LowLevel.ckbytelist} type, usable
         as a list containing only bytes.
         You can easly convert it to a binary string with::
             ''.join(chr(i) for i in ckbytelistVariable)
-        
+
         """
         valTemplate = PyKCS11.LowLevel.ckattrlist(len(attr))
         for x in xrange(len(attr)):
@@ -838,13 +868,13 @@ class Session(object):
 
         return res
 
-    def getAttributeValue_fragmented(self, obj_id, attr, allAsBinary = False):
+    def getAttributeValue_fragmented(self, obj_id, attr, allAsBinary=False):
         """
         Same as L{getAttributeValue} except that when some attribute
         is sensitive or unknown an empty value (None) is retruned.
-        
+
         Note: this is achived getting attributes one by one.
-        
+
         @see: L{getAttributeValue}
         """
         # some attributes does not exists or is sensitive
@@ -868,7 +898,7 @@ class Session(object):
             rv = self.lib.C_GetAttributeValue(self.session, obj_id, valTemplate)
             if rv != CKR_OK:
                 raise PyKCS11Error(rv)
-            
+
             if (allAsBinary):
                 res.append(valTemplate[0].GetBin())
             elif valTemplate[0].IsNum():
@@ -884,6 +914,37 @@ class Session(object):
 
         return res
 
+    def seedRandom(self, seed):
+        """
+        C_SeedRandom
+
+        @param seed: seed material
+        @type seed: iterable
+        """
+        low_seed = PyKCS11.LowLevel.ckbytelist(len(seed))
+        for c in xrange(len(seed)):
+            low_seed.append(seed[c])
+        rv = self.lib.C_SeedRandom(self.session, low_seed)
+        if rv != CKR_OK:
+            raise PyKCS11Error(rv)
+
+    def generateRandom(self, size=16):
+        """
+        C_GenerateRandom
+
+        @param size: number of random bytes to get
+        @type size: integer
+
+        @note: the returned value is an istance of L{LowLevel.ckbytelist}.
+        You can easly convert it to a binary string with::
+            ''.join(chr(i) for i in ckbytelistSignature)
+        """
+        low_rand = PyKCS11.LowLevel.ckbytelist(size)
+        rv = self.lib.C_GenerateRandom(self.session, low_rand)
+        if rv != CKR_OK:
+            raise PyKCS11Error(rv)
+        return low_rand
+
 if __name__ == "__main__":
     # sample test/debug code
     p = PyKCS11Lib()
@@ -901,10 +962,12 @@ if __name__ == "__main__":
     print "getSlotList"
     s = p.getSlotList()
     print "slots:", s
+    slot = s[1]
+    print "using slot:", slot
 
     print
     print "getSlotInfo"
-    i = p.getSlotInfo(s[0])
+    i = p.getSlotInfo(slot)
     print "slotDescription:", i.slotDescription.strip()
     print "manufacturerID:", i.manufacturerID
     print "flags:", i.flags
@@ -914,7 +977,7 @@ if __name__ == "__main__":
 
     print
     print "getTokenInfo"
-    t = p.getTokenInfo(s[0])
+    t = p.getTokenInfo(slot)
     print "label:", t.label
     print "manufacturerID:", t.manufacturerID
     print "model:", t.model
@@ -937,7 +1000,7 @@ if __name__ == "__main__":
 
     print
     print "openSession"
-    se = p.openSession(s[0])
+    se = p.openSession(slot)
 
     print
     print "sessionInfo"
@@ -949,9 +1012,19 @@ if __name__ == "__main__":
     print "flags:", si.flags2text()
     print "ulDeviceError:", si.ulDeviceError
 
+    print
+    print "seedRandom"
+    try:
+        se.seedRandom([1, 2, 3, 4])
+    except PyKCS11Error, e:
+        print e
+    print "generateRandom"
+    rand = se.generateRandom()
+    print " ".join("%02X" % i for i in rand)
+
     print
     print "login"
-    se.login(pin = "12345678")
+    se.login(pin="12345678")
 
     print
     print "sessionInfo"
@@ -981,4 +1054,3 @@ if __name__ == "__main__":
     print
     print "closeSession"
     se.closeSession()
-
