@@ -17,16 +17,17 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 import PyKCS11
-import getopt, sys
+import getopt
+import sys
 import platform
 
 # from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/142812
 # Title: Hex dumper
 # Submitter: Sebastien Keim (other recipes)
 # Last Updated: 2002/08/05
-# Version no: 1.0 
+# Version no: 1.0
+
 
-FILTER=''.join([(len(repr(chr(x)))==3) and chr(x) or '.' for x in range(256)])
 def hexx(intval):
     x = hex(intval)[2:]
     if (x[-1:].upper() == 'L'):
@@ -35,16 +36,20 @@ def hexx(intval):
         return "0%s" % x
     return x
 
-def dump(src, length = 8):
-    N=0; result=''
+
+def dump(src, length=8):
+    FILTER = ''.join([(len(repr(chr(x))) == 3) and chr(x) or '.' for x in range(256)])
+    N = 0
+    result = ''
     while src:
-        s,src = src[:length],src[length:]
-        hexa = ' '.join(["%02X"%ord(x) for x in s])
+        s, src = src[:length], src[length:]
+        hexa = ' '.join(["%02X" % ord(x) for x in s])
         s = s.translate(FILTER)
-        result += "%04X   %-*s   %s\n" % (N, length*3, hexa, s)
+        result += "%04X   %-*s   %s\n" % (N, length * 3, hexa, s)
         N += length
     return result
 
+
 def usage():
     print "Usage:", sys.argv[0],
     print "[-p pin][--pin=pin]",
@@ -77,7 +82,7 @@ for o, a in opts:
         sign = True
     elif o in ("-d", "--decrypt"):
         decrypt = True
-        
+
 red = blue = magenta = normal = ""
 if sys.stdout.isatty() and platform.system().lower() != 'windows':
     red = "\x1b[01;31m"
@@ -85,7 +90,7 @@ if sys.stdout.isatty() and platform.system().lower() != 'windows':
     magenta = "\x1b[35m"
     normal = "\x1b[0m"
 
-format_long   = magenta + "  %s:" + blue + " %s (%s)" + normal
+format_long = magenta + "  %s:" + blue + " %s (%s)" + normal
 format_binary = magenta + "  %s:" + blue + " %d bytes" + normal
 format_normal = magenta + "  %s:" + blue + " %s" + normal
 
@@ -114,13 +119,13 @@ for s in slots:
         print "Opened session 0x%08X" % session.session.value()
         if pin_available:
             try:
-                session.login(pin = pin)
+                session.login(pin=pin)
             except:
                 print "login failed, exception:", str(sys.exc_info()[1])
 
         objects = session.findObjects()
         print
-        print "Found %d objects: %s" % (len(objects), map(lambda x:x.value(), objects))
+        print "Found %d objects: %s" % (len(objects), map(lambda x: "0x%08X" % x.value(), objects))
 
         all_attributes = PyKCS11.CKA.keys()
         # remove the CKR_ATTRIBUTE_SENSITIVE attributes since we can't get
@@ -148,9 +153,9 @@ for s in slots:
                     ex = eval('0x%s' % ''.join(chr(c) for c in e).encode('hex'))
                 if sign:
                     try:
-                        toSign="12345678901234567890" # 20 bytes, SHA1 digest
+                        toSign = "12345678901234567890"  # 20 bytes, SHA1 digest
                         print "* Signing with object 0x%08X following data: %s" % (o.value(), toSign)
-                        signature = session.sign(o, toSign) 
+                        signature = session.sign(o, toSign)
                         s = ''.join(chr(c) for c in signature).encode('hex')
                         sx = eval('0x%s' % s)
                         print "Signature:"
@@ -161,8 +166,11 @@ for s in slots:
                             print dump(''.join(map(chr, m)), 16)
                             print "Exponent:"
                             print dump(''.join(map(chr, e)), 16)
-                            decrypted = pow(sx, ex, mx) # RSA
-                            if toSign == hexx(decrypted).decode('hex')[-20:]:
+                            decrypted = pow(sx, ex, mx)  # RSA
+                            print "Decrypted:"
+                            d = hexx(decrypted).decode('hex')
+                            print dump(d, 16)
+                            if toSign == d[-20:]:
                                 print "*** signature VERIFIED!\n"
                             else:
                                 print "*** signature NOT VERIFIED; decrypted value:"
@@ -174,13 +182,13 @@ for s in slots:
                 if decrypt:
                     if m and e:
                         try:
-                            toEncrypt="12345678901234567890"
+                            toEncrypt = "12345678901234567890"
                             # note: PKCS1 BT2 padding should be random data,
                             # but this is just a test and we use 0xFF...
-                            padded = "\x02%s\x00%s" % ("\xFF" * (128 - (len(toEncrypt)) -2), toEncrypt)
+                            padded = "\x00\x02%s\x00%s" % ("\xFF" * (128 - (len(toEncrypt)) - 3), toEncrypt)
                             print "* Decrypting with 0x%08X following data: %s" % (o.value(), toEncrypt)
-                            print "padded:\n",  dump(padded, 16)
-                            encrypted = pow(eval('0x%sL' % padded.encode('hex')), ex, mx) # RSA
+                            print "padded:\n", dump(padded, 16)
+                            encrypted = pow(eval('0x%sL' % padded.encode('hex')), ex, mx)  # RSA
                             encrypted1 = hexx(encrypted).decode('hex')
                             print "encrypted:\n", dump(encrypted1, 16)
                             decrypted = session.decrypt(o, encrypted1)
@@ -194,7 +202,7 @@ for s in slots:
                             print "Decrypt failed, exception:", str(sys.exc_info()[1])
                     else:
                         print "ERROR: Private key don't have MODULUS/PUBLIC_EXP"
-                    
+
             print "Dumping attributes:"
             for q, a in zip(all_attributes, attributes):
                 if a == None:
@@ -228,4 +236,3 @@ for s in slots:
 
     except PyKCS11.PyKCS11Error, e:
         print "Error:", e
-
