@@ -751,7 +751,7 @@ CK_RV CPKCS11Lib::C_Verify(
 	CK_ULONG ulInDataLen = 0;
 	CK_BYTE* pInData = Vector2Buffer(inData, ulInDataLen);
 	CK_ULONG ulInSignatureLen = 0;
-	CK_BYTE* pInSignature = Vector2Buffer(inData, ulInDataLen);
+	CK_BYTE* pInSignature = Vector2Buffer(inSignature, ulInSignatureLen);
 
 	rv = m_pFunc->C_Verify(hSession, pInData, ulInDataLen, pInSignature,
 		ulInSignatureLen);
@@ -849,7 +849,7 @@ CK_RV CPKCS11Lib::C_GenerateKeyPair(
 	if (pPrivateKeyTemplate)
 		DestroyTemplate(pPrivateKeyTemplate, ulPrivateKeyAttributeCount);
 	outhPublicKey = static_cast<CK_OBJECT_HANDLE>(hPublicKey);
-	outhPublicKey = static_cast<CK_OBJECT_HANDLE>(hPrivateKey);
+	outhPrivateKey = static_cast<CK_OBJECT_HANDLE>(hPrivateKey);
 	CPKCS11LIB_EPILOGUE;
 	return rv;
 }
@@ -929,12 +929,14 @@ CK_RV CPKCS11Lib::C_SeedRandom(
 
 CK_RV CPKCS11Lib::C_GenerateRandom(
 	CK_SESSION_HANDLE hSession,
-	vector<unsigned char> RandomData)
+	vector<unsigned char> &RandomData)
 {
 	CPKCS11LIB_PROLOGUE(C_GenerateRandom);
 	CK_ULONG ulOutDataLen = 0;
 	CK_BYTE* pOutData = Vector2Buffer(RandomData, ulOutDataLen);
 	rv = m_pFunc->C_GenerateRandom(hSession, pOutData, ulOutDataLen);
+    if (CKR_OK == rv)
+		Buffer2Vector(pOutData, ulOutDataLen, RandomData, true);
 	if (pOutData)
 		delete []pOutData;
 	CPKCS11LIB_EPILOGUE;
